/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package benchmark.ring;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Collections;
import java.util.Map;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.control.activity.Util;
import org.objectweb.dream.control.activity.task.AbstractTask;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageTypeImpl;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * 
 */
public class RingStatImpl extends AbstractComponent
    implements
      Push,
      PayLoadSizeAttributeController
{

  protected boolean        messageSent      = false;
  protected long           nbRing           = 0;
  protected long           nbRingSinceLast  = 0;
  protected int            nbDisplay        = 0;
  protected long           lastTimestamp    = 0;
  protected long           firstTimestamp   = 0;
  protected double         last5sum         = 0;
  protected double         last10sum        = 0;
  protected double         last15sum        = 0;
  protected double[]       last15Rates      = new double[15];
  protected int            last15RatesIndex = 0;
  protected NumberFormat   rateFormatter    = new DecimalFormat(".00"); ;

  // ---------------------------------------------------------------------------
  // Attribute fields
  // ---------------------------------------------------------------------------

  protected int            payLoadSize      = -1;

  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------

  protected Push           outPushItf;
  protected MessageManager messageManagerItf;

  // ---------------------------------------------------------------------------
  // Inner task class implementation
  // ---------------------------------------------------------------------------

  protected class StatPrinterTask extends AbstractTask
  {

    /** Constructor */
    public StatPrinterTask()
    {
      super("StatPrinterTask");
    }

    /**
     * @see org.objectweb.deployment.scheduling.core.api.Task#execute(Object)
     */
    public Object execute(Object hints) throws InterruptedException
    {
      if (!messageSent)
      {
        Message message = messageManagerItf.createMessage(new MessageTypeImpl(
            PayLoadChunk.DEFAULT_NAME, PayLoadChunk.TYPE));
        PayLoadChunk chunk = (PayLoadChunk) message
            .getChunk(PayLoadChunk.DEFAULT_NAME);
        chunk.setPayLoadSize(payLoadSize);
        synchronized (RingStatImpl.this)
        {
          try
          {
            outPushItf.push(message, null);
          }
          catch (PushException e)
          {
            logger.log(BasicLevel.ERROR, "Unable to push message", e);
          }
        }
        messageSent = true;
      }
      Thread.sleep(5000);

      if (nbDisplay % 50 == 0)
      {
        // print header.
        System.out
            .println("|-------|------------|------------|------------|------------|------------|-------------|");
        System.out
            .println("|       |       last |     last 5 |    last 10 |    last 15 |     global |     nb ring |");
        System.out
            .println("|-------|------------|------------|------------|------------|------------|-------------|");
      }
      nbDisplay++;
      long nextTimestamp = System.currentTimeMillis();
      if (lastTimestamp == 0)
      {
        lastTimestamp = firstTimestamp;
      }

      nbRing += nbRingSinceLast;
      double rate = nbRingSinceLast * 1000.0 / (nextTimestamp - lastTimestamp);
      double globalRate = nbRing * 1000.0 / (nextTimestamp - firstTimestamp);
      last5sum += rate;
      last10sum += rate;
      last15sum += rate;
      if (nbDisplay >= 5)
      {
        last5sum -= last15Rates[(last15RatesIndex + 10) % 15];
        if (nbDisplay >= 10)
        {
          last10sum -= last15Rates[(last15RatesIndex + 5) % 15];
          if (nbDisplay >= 15)
          {
            last15sum -= last15Rates[last15RatesIndex];
          }
        }
      }
      last15Rates[last15RatesIndex] = rate;
      last15RatesIndex = (last15RatesIndex + 1) % 15;
      nbRingSinceLast = 0;
      lastTimestamp = nextTimestamp;

      StringBuffer buffer = new StringBuffer("| ");
      formatLong(buffer, nbDisplay, 5).append(" | ");
      formatDouble(buffer, rate, 10).append(" | ");
      if (nbDisplay < 5)
      {
        buffer.append("           |            |            | ");
      }
      else
      {
        formatDouble(buffer, last5sum / 5, 10).append(" | ");
        if (nbDisplay < 10)
        {
          buffer.append("           |            | ");
        }
        else
        {
          formatDouble(buffer, last10sum / 10, 10).append(" | ");
          if (nbDisplay < 15)
          {
            buffer.append("           | ");
          }
          else
          {
            formatDouble(buffer, last15sum / 15, 10).append(" | ");
          }
        }
      }
      formatDouble(buffer, globalRate, 10).append(" | ");
      formatLong(buffer, nbRing, 11).append(" |");
      System.out.println(buffer);

      return EXECUTE_AGAIN;
    }
  }

  // -------------------------------------------------------------------------
  // Overriden methods
  // -------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.AbstractComponent#beforeFirstStart(org.objectweb.fractal.api.Component)
   */
  protected void beforeFirstStart(Component componentItf)
      throws IllegalLifeCycleException
  {
    try
    {
      Util.addTask(componentItf, new StatPrinterTask(), Collections.EMPTY_MAP);
    }
    catch (Exception e)
    {
      throw new IllegalLifeCycleException("Can't add task");
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Push interface
  // ---------------------------------------------------------------------------

  /**
   * @see Push#push(Message, Map)
   */
  public void push(Message message, Map context) throws PushException
  {
    synchronized (this)
    {
      if (firstTimestamp == 0)
      {
        firstTimestamp = System.currentTimeMillis();
      }
      nbRingSinceLast++;
      outPushItf.push(message, context);
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the AttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see PayLoadSizeAttributeController#getPayLoadSize()
   */
  public int getPayLoadSize()
  {
    return payLoadSize;
  }

  /**
   * @see PayLoadSizeAttributeController#setPayLoadSize(int)
   */
  public void setPayLoadSize(int nbByte)
  {
    payLoadSize = nbByte;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{MessageManager.ITF_NAME, Push.OUT_PUSH_ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
    else if (clientItfName.equals(Push.OUT_PUSH_ITF_NAME))
    {
      outPushItf = (Push) serverItf;
    }
  }

  // ---------------------------------------------------------------------------
  // Utility methods
  // ---------------------------------------------------------------------------

  private StringBuffer formatLong(StringBuffer buffer, long value, int nbChar)
  {
    String s = Long.toString(value);
    for (int i = s.length(); i < nbChar; i++)
    {
      buffer.append(' ');
    }
    return buffer.append(s);
  }

  private StringBuffer formatDouble(StringBuffer buffer, double value,
      int nbChar)
  {
    String s = rateFormatter.format(value);
    for (int i = s.length(); i < nbChar; i++)
    {
      buffer.append(' ');
    }
    return buffer.append(s);
  }

}