/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.channel;

import java.io.IOException;
import java.net.InetAddress;
import java.util.HashMap;
import java.util.Map;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * This component manage a cache of {@link SocketState}.
 */
public class IPSocketManagerCachingImpl extends AbstractComponent
    implements
      IPSocketManager,
      IPSocketManagerCachingAttributeController
{

  /**
   * The name of the client interface used to retreive a socket when it can't be
   * found in the cache.
   */
  public static final String DELEGATE_IP_SOCKET_MANAGER = "delegate-ip-socket-manager";

  protected CacheEntry       reusableKey                = new CacheEntry();
  protected Map              cache                      = new HashMap();
  protected Map              reversCache                = new HashMap();
  protected CacheEntry       lruHead                    = null;
  protected CacheEntry       lruTail                    = null;

  protected boolean          blocking                   = true;

  // ---------------------------------------------------------------------------
  // Attribute fields
  // ---------------------------------------------------------------------------

  protected int              nbMaxSocket;

  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------

  protected IPSocketManager  delegateSocketManagerItf;

  // ---------------------------------------------------------------------------
  // Implementation of the IPSocketManager interface
  // ---------------------------------------------------------------------------

  /**
   * @see IPSocketManager#getSocket(InetAddress, int)
   */
  public SocketState getSocket(InetAddress address, int port)
      throws IOException, InterruptedException
  {
    CacheEntry ce;
    synchronized (cache)
    {
      if (logger.isLoggable(BasicLevel.DEBUG))
      {
        logger.log(BasicLevel.DEBUG, "Get socket in cache for " + address + ":"
            + port);
      }
      reusableKey.inetAddress = address;
      reusableKey.port = port;
      ce = (CacheEntry) cache.get(reusableKey);
      if (ce != null)
      {
        logger.log(BasicLevel.DEBUG, "Socket found in cache");
        while (ce.inUse)
        {
          if (blocking)
          {
            cache.wait();
          }
          else
          {
            throw new IOException("Socket already in use");
          }
        }
        if (ce.socketState != null)
        {
          if (ce.socketState.isClosed())
          {
            logger.log(BasicLevel.DEBUG, "Socket is closed");
            ce.cacheRemove();
          }
          else
          {
            ce.inUse = true;
            ce.lruSetAtfirst();
            return ce.socketState;
          }
        }
      }

      // open new socket.
      if (cache.size() >= nbMaxSocket)
      {
        // remove last from lru
        if (lruTail.inUse)
        {
          throw new IOException(
              "Can't add new socket, cache full and every sockets are in use.");
        }

        lruTail.cacheRemove();
      }

      ce = new CacheEntry();
      ce.inetAddress = address;
      ce.port = port;
      ce.inUse = true;
      cache.put(ce, ce);
    }
    // get new socket outside from synchronize
    if (logger.isLoggable(BasicLevel.DEBUG))
    {
      logger.log(BasicLevel.DEBUG, "Create new socket for " + address + ":"
          + port);
    }
    SocketState socketState = delegateSocketManagerItf.getSocket(address, port);
    synchronized (cache)
    {
      logger.log(BasicLevel.DEBUG, "new socket created.");
      ce.socketState = socketState;
      reversCache.put(socketState, ce);
      return socketState;
    }
  }

  /**
   * @see IPSocketManager#releaseSocket(SocketState, boolean)
   */
  public void releaseSocket(SocketState socketState, boolean error)
  {
    synchronized (cache)
    {
      CacheEntry ce = (CacheEntry) reversCache.get(socketState);
      if (ce == null)
      {
        throw new IllegalArgumentException("Unknown SocketState");
      }
      if (error)
      {
        ce.cacheRemove();
        ce.socketState = null;
      }
      else
      {
        ce.inUse = false;
      }
      cache.notifyAll();
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the AttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see IPSocketManagerCachingAttributeController#getNbMaxSocket()
   */
  public int getNbMaxSocket()
  {
    return nbMaxSocket;
  }

  /**
   * @see IPSocketManagerCachingAttributeController#setNbMaxSocket(int)
   */
  public void setNbMaxSocket(int nbMaxSocket)
  {
    this.nbMaxSocket = nbMaxSocket;
  }

  /**
   * @see org.objectweb.dream.channel.IPSocketManagerCachingAttributeController#setBlocking(boolean)
   */
  public void setBlocking(boolean blocking)
  {
    this.blocking = blocking;
  }

  /**
   * @see org.objectweb.dream.channel.IPSocketManagerCachingAttributeController#getBlocking()
   */
  public boolean getBlocking()
  {
    return blocking;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{DELEGATE_IP_SOCKET_MANAGER};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(DELEGATE_IP_SOCKET_MANAGER))
    {
      delegateSocketManagerItf = (IPSocketManager) serverItf;
    }
  }

  // ---------------------------------------------------------------------------
  // Utility inner class
  // ---------------------------------------------------------------------------

  protected class CacheEntry
  {

    protected InetAddress inetAddress;
    protected int         port;
    protected SocketState socketState;

    protected CacheEntry  next  = null;
    protected CacheEntry  prev  = null;
    protected boolean     inUse = false;

    protected void cacheRemove()
    {
      if (cache.remove(this) == null)
      {
        Error.bug(logger);
      }
      if (reversCache.remove(socketState) == null)
      {
        Error.bug(logger);
      }
      lruRemove();
    }

    protected void lruRemove()
    {
      if (next == null)
      {
        // this == lruTail
        if (prev == null)
        {
          // lru contains only this
          lruHead = null;
          lruTail = null;
        }
        else
        {
          lruTail = prev;
          lruTail.next = null;
        }
      }
      else
      {
        // next non null
        next.prev = prev;
        if (prev == null)
        {
          // this == lruHead
          lruHead = next;
        }
        else
        {
          prev.next = next;
        }
      }
      next = null;
      prev = null;
    }

    protected void lruSetAtfirst()
    {
      if (prev == null)
      {
        // already first
        return;
      }
      prev.next = next;
      if (next == null)
      {
        // this == lruTail
        lruTail = prev;
      }
      else
      {
        next.prev = prev;
      }
      lruHead.prev = this;
      next = lruHead;
      prev = null;
      lruHead = this;
    }

    protected void lruAddAtFirst()
    {
      if (lruHead == null)
      {
        // lru empty
        lruHead = this;
        lruTail = this;
        return;
      }
      lruHead.prev = this;
      next = lruHead;
      prev = null;
      lruHead = this;
    }

    /**
     * @see Object#equals(Object)
     */
    public boolean equals(Object obj)
    {
      final CacheEntry key = (CacheEntry) obj;
      return (inetAddress == null ? key.inetAddress == null : inetAddress
          .equals(key.inetAddress))
          && port == key.port;
    }

    /**
     * @see Object#hashCode()
     */
    public int hashCode()
    {
      return inetAddress == null ? inetAddress.hashCode() + port : port;
    }
  }

}