/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.channel;

import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.control.activity.Util;
import org.objectweb.dream.control.activity.task.AbstractTask;
import org.objectweb.dream.control.activity.task.Task;
import org.objectweb.dream.control.activity.task.TaskController;
import org.objectweb.dream.control.activity.task.thread.ThreadPoolController;
import org.objectweb.dream.control.activity.task.thread.ThreadPoolOverflowException;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.julia.control.lifecycle.ChainedIllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Opened socket manager component that lanage multiple opened sockets at a
 * time. Each opened socket is executed by its own task.
 */
public class OpenedSocketManagerMultiPersistentImpl extends AbstractComponent
    implements
      OpenedSocket,
      OpenedSocketManagerMultiAttributeController
{

  protected Task                 inTask              = new InTask();

  protected ThreadPoolController threadPoolController;

  protected LinkedList           availableSocketList = new LinkedList();
  protected int                  maxOpenedSocket;
  protected int                  nbOpenedSocket      = 0;

  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------

  protected OpenedSocket         delegateOpenedSocketItf;

  // ---------------------------------------------------------------------------
  // Inner task class implementation
  // ---------------------------------------------------------------------------

  protected class InTask extends AbstractTask
  {

    /** Constructor */
    public InTask()
    {
      super("ChannelIn-reader-task");
    }

    /**
     * @see Task#execute(Object)
     */
    public Object execute(Object hints) throws InterruptedException
    {
      SocketState socketState;
      synchronized (availableSocketList)
      {
        socketState = (SocketState) availableSocketList.removeFirst();
      }
      logger.log(BasicLevel.DEBUG, "ChannelIn-reader-task : got a connection");
      if (socketState.isClosed())
      {
        stopThread(socketState);
        logger.log(BasicLevel.DEBUG,
            "ChannelIn-reader-task : connection closed, stop thread");
        return STOP_EXECUTING;
      }

      try
      {
        // call delegate
        delegateOpenedSocketItf.openedSocket(socketState);
      }
      catch (IOException e)
      {
        logger
            .log(
                BasicLevel.WARN,
                "ChannelIn-reader-task : I/O error while receiving message, close connection",
                e);
        stopThread(socketState);
        return STOP_EXECUTING;
      }

      synchronized (availableSocketList)
      {
        availableSocketList.add(socketState);
      }
      return EXECUTE_AGAIN;
    }

    protected void stopThread(SocketState socketState)
    {
      synchronized (availableSocketList)
      {
        nbOpenedSocket--;
        // notify connection task.
        availableSocketList.notify();
      }
      socketState.close();
    }
  }

  // -------------------------------------------------------------------------
  // Overriden methods
  // -------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.AbstractComponent#beforeFirstStart(org.objectweb.fractal.api.Component)
   */
  protected void beforeFirstStart(Component componentItf)
      throws IllegalLifeCycleException
  {
    try
    {
      Map hints = new HashMap();
      hints.put("thread", "pool");
      logger.log(BasicLevel.DEBUG, "Initial Max connection=" + maxOpenedSocket);
      Util.addTask(componentItf, inTask, hints);
    }
    catch (Exception e)
    {
      throw new IllegalLifeCycleException("Can't add task");
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of OpenedSocket interface
  // ---------------------------------------------------------------------------

  /**
   * @see OpenedSocket#openedSocket(SocketState)
   */
  public void openedSocket(SocketState socket) throws IOException,
      InterruptedException
  {
    synchronized (availableSocketList)
    {
      nbOpenedSocket++;
      availableSocketList.add(socket);
      if (threadPoolController != null)
      {
        try
        {
          logger.log(BasicLevel.DEBUG, "add a thread in thread pool");
          threadPoolController.addThreads(1);
        }
        catch (ThreadPoolOverflowException e)
        {
          logger.log(BasicLevel.WARN, "Unable to add reader thread", e);
        }
        catch (IllegalLifeCycleException e)
        {
          Error.bug(logger, e);
        }
      }
      // if maxConnection reach, wait.
      while (nbOpenedSocket >= maxOpenedSocket)
      {
        availableSocketList.wait();
      }
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of AttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see OpenedSocketManagerMultiAttributeController#getMaxOpenedSocket()
   */
  public int getMaxOpenedSocket()
  {
    return maxOpenedSocket;
  }

  /**
   * @see OpenedSocketManagerMultiAttributeController#setMaxOpenedSocket(int)
   */
  public void setMaxOpenedSocket(int maxOpenedSocket)
  {
    this.maxOpenedSocket = maxOpenedSocket;
    if (threadPoolController != null)
    {
      threadPoolController.setCapacity(maxOpenedSocket);
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of LifeCycleController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.AbstractComponent#startFc()
   */
  public void startFc() throws IllegalLifeCycleException
  {
    super.startFc();
    synchronized (availableSocketList)
    {
      try
      {
        TaskController tc = (TaskController) weaveableC
            .getFcInterface("task-controller");
        threadPoolController = (ThreadPoolController) tc.getTaskControl(inTask);
        threadPoolController.setCapacity(getMaxOpenedSocket());
        threadPoolController.addThreads(availableSocketList.size());
      }
      catch (Exception e)
      {
        throw new ChainedIllegalLifeCycleException(e, null,
            "An error occurs while retreiving task control interface");
      }
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
   */
  public void stopFc() throws IllegalLifeCycleException
  {
    super.stopFc();
    threadPoolController = null;
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{OpenedSocket.ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(OpenedSocket.ITF_NAME))
    {
      delegateOpenedSocketItf = (OpenedSocket) serverItf;
    }
  }
}