/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.channel;

import java.io.IOException;
import java.net.BindException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;

import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Wraps TCP ServerSocket.
 */
public class TCPAcceptSocketImpl extends AbstractAcceptSocketImpl
    implements
      TCPAcceptSocketAttributeController
{

  /** The {@link ServerSocket}used by this component */
  protected ServerSocket listen;

  // ---------------------------------------------------------------------------
  // Attributes fields
  // ---------------------------------------------------------------------------

  protected int          listeningPort = -1;
  protected int          openRetry     = OPEN_RETRY_DEFAULT;
  protected boolean      tcpNoDelay    = true;
  protected int          soTimeout     = 0;
  protected int          soLinger      = 60;

  // ---------------------------------------------------------------------------
  // Implementation of abstract method
  // ---------------------------------------------------------------------------

  /**
   * @see AbstractAcceptSocketImpl#acceptSocket()
   */
  protected SocketState acceptSocket() throws IOException
  {
    if (listen == null || listen.isClosed())
    {
      logger.log(BasicLevel.DEBUG, "ServerSocket closed, create new one");
      listen = createServerSocket();
    }
    logger.log(BasicLevel.DEBUG, "waiting connection");
    Socket socket = listen.accept();
    setSocketOption(socket);
    SocketState socketState = new SocketStateImpl();
    socketState.setSocket(socket);
    return socketState;
  }

  // ---------------------------------------------------------------------------
  // Attribute controller interface
  // ---------------------------------------------------------------------------

  /**
   * @see TCPAcceptSocketAttributeController#getListeningPort()
   */
  public int getListeningPort()
  {
    return listeningPort;
  }

  /**
   * @see TCPAcceptSocketAttributeController#setListeningPort(int)
   */
  public void setListeningPort(int port) throws IllegalLifeCycleException
  {
    if (getFcState() == STARTED)
    {
      throw new IllegalLifeCycleException(
          "The component is started, can't change the listeningPort attribute");
    }
    listeningPort = port;
  }

  /**
   * @see TCPAcceptSocketAttributeController#getOpenRetry()
   */
  public int getOpenRetry()
  {
    return openRetry;
  }

  /**
   * @see TCPAcceptSocketAttributeController#setOpenRetry(int)
   */
  public void setOpenRetry(int retry)
  {
    openRetry = retry;
  }

  /**
   * @see TCPAcceptSocketAttributeController#getSoLinger()
   */
  public int getSoLinger()
  {
    return soLinger;
  }

  /**
   * @see TCPAcceptSocketAttributeController#setSoLinger(int)
   */
  public void setSoLinger(int timeout)
  {
    soLinger = timeout;
  }

  /**
   * @see TCPAcceptSocketAttributeController#getSoTimeout()
   */
  public int getSoTimeout()
  {
    return soTimeout;
  }

  /**
   * @see TCPAcceptSocketAttributeController#setSoTimeout(int)
   */
  public void setSoTimeout(int timeout)
  {
    soTimeout = timeout;
  }

  /**
   * @see TCPAcceptSocketAttributeController#getTcpNoDelay()
   */
  public boolean getTcpNoDelay()
  {
    return tcpNoDelay;
  }

  /**
   * @see TCPAcceptSocketAttributeController#setTcpNoDelay(boolean)
   */
  public void setTcpNoDelay(boolean tcpNoDelay)
  {
    this.tcpNoDelay = tcpNoDelay;
  }

  //---------------------------------------------------------------------------
  // Overriden Lifecycle controller methods
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
   */
  public void startFc() throws IllegalLifeCycleException
  {
    if (listeningPort < 0)
    {
      throw new IllegalLifeCycleException("Unset attribute : listeningPort");
    }
    super.startFc();
  }

  /**
   * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
   */
  public void stopFc() throws IllegalLifeCycleException
  {
    if (listen != null && !listen.isClosed())
    {
      try
      {
        listen.close();
      }
      catch (IOException e)
      {
        logger.log(BasicLevel.WARN,
            "An error occured while closing server socket", e);
      }
      listen = null;
    }
    super.stopFc();
  }

  // ---------------------------------------------------------------------------
  // Utility Methods
  // ---------------------------------------------------------------------------

  protected ServerSocket createServerSocket() throws IOException
  {
    for (int i = 0;; i++)
    {
      try
      {
        return new ServerSocket(listeningPort);
      }
      catch (BindException exc)
      {
        if (i > openRetry)
        {
          throw exc;
        }
        try
        {
          Thread.sleep(i * 250);
        }
        catch (InterruptedException ignored)
        {
          // ignored
        }
      }
    }
  }

  protected void setSocketOption(Socket sock) throws SocketException
  {
    sock.setTcpNoDelay(tcpNoDelay);
    sock.setSoTimeout(soTimeout);
    if (soLinger >= 0)
    {
      sock.setSoLinger(true, soLinger);
    }
    else
    {
      sock.setSoLinger(false, 0);
    }
  }

}