/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.utobcast;

import java.util.Map;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.dream.protocol.utobcast.message.UPDChunk;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Implementation of the UPD component. This component handles UPD messages.
 * Upon message reception, it updates the <code>PendingMessages</code> queue,
 * the <code>SequenceNumber</code> component, and the
 * <code>ProcessMembership</code> component.
 */
public class UPDImpl extends AbstractComponent implements Push
{

  // ---------------------------------------------------------------------------
  // Fields of this class
  // ---------------------------------------------------------------------------

  /**
   * The commonly used name to refer to the <code>pendingMessageInItf</code>
   * interface.
   */
  public static final String  PENDING_MESSAGES_IN_ITF_NAME = "pending-messages-in";

  /** The interface to store messages in the <code>PendingMessage</code> queue. */
  protected Push              pendingMessagesInItf;

  /** The interface to retrieve process membership information. */
  protected ProcessMembership processMembershipItf;

  /** The interface to get/set the sequence number. */
  protected SequenceNumber    sequenceNumberItf;

  /** The interface to manage message lifecycle. */
  protected MessageManager    messageManagerItf;

  // ---------------------------------------------------------------------------
  // Constructor
  // ---------------------------------------------------------------------------

  /**
   * Constructor.
   */
  public UPDImpl()
  {
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Push interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.Push#push(org.objectweb.dream.message.Message,
   *      java.util.Map)
   */
  public void push(Message message, Map context) throws PushException
  {
    UPDChunk chunk = (UPDChunk) message.getChunk(UPDChunk.DEFAULT_NAME);
    processMembershipItf.setOtherProcesses(chunk.getProcesses());
    sequenceNumberItf.setSequenceNumber(chunk.getSequenceNumber());
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
   *      java.lang.Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(PENDING_MESSAGES_IN_ITF_NAME))
    {
      pendingMessagesInItf = (Push) serverItf;
    }
    else if (clientItfName.equals(SequenceNumber.ITF_NAME))
    {
      sequenceNumberItf = (SequenceNumber) serverItf;
    }
    else if (clientItfName.equals(ProcessMembership.ITF_NAME))
    {
      processMembershipItf = (ProcessMembership) serverItf;
    }
    else if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }

  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{PENDING_MESSAGES_IN_ITF_NAME, SequenceNumber.ITF_NAME,
        ProcessMembership.ITF_NAME, MessageManager.ITF_NAME};
  }

}