/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.protocol.utobcast.message;

import org.objectweb.dream.message.ChunkType;
import org.objectweb.dream.message.ChunkTypeImpl;
import org.objectweb.dream.protocol.Process;
import org.objectweb.dream.queue.SequenceNumberChunk;

/**
 * Chunk interface for UTOBcast protocol messages. This interface allows
 * retrieving the type of the message that contains it: DAT, ACK, REP, UTO, UPD,
 * or BAK.
 */
public interface UTOBcastChunk extends SequenceNumberChunk
{
  // TODO extends SequenceNumberChunk is not very "exo" !... Could be put in
  // another chunk

  /** The default name for chunks of this type. */
  String    DEFAULT_NAME = "utoBcastChunk";

  /** The Chunk type of chunks that implements this interface. */
  ChunkType TYPE         = new ChunkTypeImpl(UTOBcastChunk.class,
                             UTOBcastChunkImpl.class);

  /**
   * The byte value for DAT message type.
   */
  byte      DAT          = 0;

  /**
   * The byte value for ACK message type.
   */
  byte      ACK          = 1;

  /**
   * The byte value for REP message type.
   */
  byte      REP          = 2;

  /**
   * The byte value for UTO message type.
   */
  byte      UTO          = 3;

  /**
   * The byte value for UPD message type.
   */
  byte      UPD          = 4;

  /**
   * The byte value for BAK message type.
   */
  byte      BAK          = 5;

  /**
   * Returns the UTOBcast type of the message that contains this chunk.
   * 
   * @return <ul>
   *         <li>0 for DAT messages</li>
   *         <li>1 for ACK messages</li>
   *         <li>2 for REP messages</li>
   *         <li>3 for UTO messages</li>
   *         <li>4 for UPD messages</li>
   *         <li>5 for BACK messages</li>
   *         </ul>
   */
  byte getUTOBcastMessageType();

  /**
   * Sets the UTOBcast type of the message that contains this chunk. This type
   * can be
   * <ul>
   * <li>0 for DAT messages</li>
   * <li>1 for ACK messages</li>
   * <li>2 for REP messages</li>
   * <li>3 for UTO messages</li>
   * <li>4 for UPD messages</li>
   * <li>5 for BACK messages</li>
   * </ul>
   * 
   * @param type the UTOBcast type to set.
   */
  void setUTOBcastMessageType(byte type);

  /**
   * Returns the process that sent this message.
   * 
   * @return the process that sent this message.
   */
  Process getProcessFrom();

  /**
   * Sets the process that sent this message.
   * 
   * @param process the process that sent this message.
   */
  void setProcessFrom(Process process);

  /**
   * Returns the process to which this message is destinated.
   * 
   * @return the process to which this message is destinated.
   */
  Process getProcessTo();

  /**
   * Sets the process to which this message is destinated.
   * 
   * @param process the process to which this message is destinated.
   */
  void setProcessTo(Process process);

}