/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package org.objectweb.dream.queue;

import java.util.Map;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.InterruptedPushException;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Abstract implementation of a Push incoming handler. It implements overflow
 * policies as defined in {@link PushQueueAttributeController}. Moreover, it
 * implements the <code>push</code> method: it checks for available space and
 * then applies appropriate policy.
 * <p>
 * Push incoming handler developers can inherit this class. They must implement
 * two methods:
 * <ul>
 * <li><code>canAdd(message)</code> returns a boolean indicating whether the
 * given message can be added into the buffer.</li>
 * <li><code>doAdd(message)</code> adds the given message into the buffer.
 * </li>
 * </ul>
 */
public abstract class AbstractPushIncomingHandlerOverflowImpl
    extends
      AbstractComponent implements Push, PushQueueAttributeController
{

  // Ids used in switch instructions
  protected static final int      BLOCK_OVERFLOW_POLICY_ID                  = 0;
  protected static final int      DROP_QUEUE_MESSAGE_OVERFLOW_POLICY_ID     = 1;
  protected static final int      DROP_FIRST_OVERFLOW_POLICY_ID             = 2;
  protected static final int      DROP_LAST_OVERFLOW_POLICY_ID              = 3;
  protected static final int      DROP_PROCESSED_MESSAGE_OVERFLOW_POLICY_ID = 4;
  protected static final int      EXCEPTION_OVERFLOW_POLICY_ID              = 5;

  protected String                overflowPolicy;
  protected short                 overflowPolicyId;

  // Client interfaces
  protected MessageManager        messageManagerItf;
  protected Buffer                bufferItf;
  protected BufferRemoveFirstLast bufferRemoveFirstLastItf;
  protected BufferAddFirstLast    bufferAddFirstLastItf;

  // ---------------------------------------------------------------------------
  // Implementation of the Push interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.Push#push(org.objectweb.dream.message.Message,
   *      java.util.Map)
   */
  public void push(Message message, Map context) throws PushException
  {
    try
    {
      if (overflowPolicyId != BLOCK_OVERFLOW_POLICY_ID && !canAdd(message))
      {
        switch (overflowPolicyId)
        {
          case DROP_QUEUE_MESSAGE_OVERFLOW_POLICY_ID : {
            messageManagerItf.deleteMessage(bufferItf.remove());
            break;
          }
          case DROP_FIRST_OVERFLOW_POLICY_ID : {
            if (bufferRemoveFirstLastItf != null)
            {
              messageManagerItf.deleteMessage(bufferRemoveFirstLastItf
                  .removeFirst());
            }
            else
            {
              throw new UnsupportedOperationException(
                  "Unable to remove first element of the queue.");
            }
            break;
          }
          case DROP_LAST_OVERFLOW_POLICY_ID : {
            if (bufferRemoveFirstLastItf != null)
            {
              messageManagerItf.deleteMessage(bufferRemoveFirstLastItf
                  .removeLast());
            }
            else
            {
              throw new UnsupportedOperationException(
                  "Unable to remove last element of the queue.");
            }
            break;
          }
          case DROP_PROCESSED_MESSAGE_OVERFLOW_POLICY_ID : {
            messageManagerItf.deleteMessage(message);
            break;
          }
          case EXCEPTION_OVERFLOW_POLICY_ID : {
            throw new BufferOverflowException("Overflow in buffer");
          }
          default : {
            Error.error("Unknown overflow policy" + overflowPolicyId, logger);
          }
        }
      }
      else
      {
        doAdd(message);
      }
    }
    catch (InterruptedException e)
    {
      throw new InterruptedPushException("Interrupted while adding a message",
          e);
    }
  }

  // ---------------------------------------------------------------------------
  // Methods that must be implemented by concrete super classes.
  // ---------------------------------------------------------------------------

  /**
   * Checks whether the given message can be added into the buffer.
   * 
   * @param message the message to be tested.
   * @return <code>true</code> if the message can be added.
   */
  protected abstract boolean canAdd(Message message);

  /**
   * Adds a message to the buffer. This method should not check if there is
   * enough available space provided it has already been done.
   * 
   * @param message the message to be added.
   * @throws InterruptedException if it is interrupted while removing the
   *           message.
   */
  protected abstract void doAdd(Message message) throws InterruptedException;

  // ---------------------------------------------------------------------------
  // Implementation of the PushQueueAttributeController interface.
  // ---------------------------------------------------------------------------

  /**
   * @see PushQueueAttributeController#getOverflowPolicy()
   */
  public String getOverflowPolicy()
  {
    return overflowPolicy;
  }

  /**
   * @see PushQueueAttributeController#setOverflowPolicy(String)
   */
  public synchronized void setOverflowPolicy(String policy)
  {
    // set overflow policy attribute reference to interface constant to test
    // policy efficiently.
    if (policy.equals(BLOCK_OVERFLOW_POLICY))
    {
      overflowPolicy = BLOCK_OVERFLOW_POLICY;
      overflowPolicyId = BLOCK_OVERFLOW_POLICY_ID;
    }
    else if (policy.equals(DROP_QUEUE_MESSAGE_OVERFLOW_POLICY))
    {
      overflowPolicy = DROP_QUEUE_MESSAGE_OVERFLOW_POLICY;
      overflowPolicyId = DROP_QUEUE_MESSAGE_OVERFLOW_POLICY_ID;
    }
    else if (policy.equals(DROP_FIRST_OVERFLOW_POLICY))
    {
      overflowPolicy = DROP_FIRST_OVERFLOW_POLICY;
      overflowPolicyId = DROP_FIRST_OVERFLOW_POLICY_ID;
    }
    else if (policy.equals(DROP_LAST_OVERFLOW_POLICY))
    {
      overflowPolicy = DROP_LAST_OVERFLOW_POLICY;
      overflowPolicyId = DROP_LAST_OVERFLOW_POLICY_ID;
    }
    else if (policy.equals(DROP_PROCESSED_MESSAGE_OVERFLOW_POLICY))
    {
      overflowPolicy = DROP_PROCESSED_MESSAGE_OVERFLOW_POLICY;
      overflowPolicyId = DROP_PROCESSED_MESSAGE_OVERFLOW_POLICY_ID;
    }
    else if (policy.equals(EXCEPTION_OVERFLOW_POLICY))
    {
      overflowPolicy = EXCEPTION_OVERFLOW_POLICY;
      overflowPolicyId = EXCEPTION_OVERFLOW_POLICY_ID;
    }
    else
    {
      throw new IllegalArgumentException("Unknown overflow policy : " + policy);
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
    else if (clientItfName.equals(Buffer.ITF_NAME))
    {
      bufferItf = (Buffer) serverItf;
    }
    else if (clientItfName.equals(BufferRemoveFirstLast.ITF_NAME))
    {
      bufferRemoveFirstLastItf = (BufferRemoveFirstLast) serverItf;
    }
    else if (clientItfName.equals(BufferAddFirstLast.ITF_NAME))
    {
      bufferAddFirstLastItf = (BufferAddFirstLast) serverItf;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{MessageManager.ITF_NAME, Buffer.ITF_NAME,
        BufferRemoveFirstLast.ITF_NAME, BufferAddFirstLast.ITF_NAME};
  }

}