/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.control.activity.task.thread;

import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Controller interface for thread pools. It allows increasing or decreasing the
 * number of threads in the pool.
 */
public interface ThreadPoolController
{

  /**
   * The default timeout.
   * 
   * @see #setWaitTimeout(long)
   */
  long DEFAULT_WAIT_TIMEOUT = 2000;

  /**
   * Sets the capacity of the pool. The capacity of the pool is the maximum
   * number of currently executing threads. If the capacity is reduced,
   * currently executing threads are not stopped. As a consequence, the number
   * of executing threads may temporarily exceed the capacity.
   * 
   * @param i the capacity of the pool.
   */
  void setCapacity(int i);

  /**
   * Returns the capacity of the pool.
   * 
   * @return the capacity of the pool.
   */
  int getCapacity();

  /**
   * Sets the timeout while a "dying" thread can be "reactivated" and reused,
   * rather than create a new thread.
   * 
   * @param millis timeout in millisecond.
   */
  void setWaitTimeout(long millis);

  /**
   * Returns the timeout while "dying" thread can be "reactivated".
   * 
   * @return timeout in millisecond.
   */
  long getWaitTimeout();
  
  /**
   * Returns the number of currently active threads.
   * 
   * @return the number of currently active threads.
   */
  int getNbActiveThreads();

  /**
   * Adds threads in the pool. If the thread pool is stopping (see
   * {@link org.objectweb.dream.control.activity.task.TaskLifeCycleController})
   * the added threads are immediatly interrupted.
   * 
   * @param i the number of thread to add.
   * @throws ThreadPoolOverflowException if
   *           <code>getNbActiveThread() + i &gt; getCapacity() </code>
   * @throws IllegalLifeCycleException if the component this interface belong
   *           has a lifecycle controller and is in the
   *           {@link org.objectweb.fractal.api.control.LifeCycleController#STOPPED}
   *           state.
   */
  void addThreads(int i) throws ThreadPoolOverflowException,
      IllegalLifeCycleException;

  /**
   * Removes threads from the pool. The threads are removed in a FIFO order.
   * This method is not synchronized with the end of the removed threads. This
   * means that the method may return before the end of the removed threads.
   * 
   * @param i the number of thread to remove.
   * @throws IllegalLifeCycleException if the component this interface belong
   *           has a lifecycle controller and is in the
   *           {@link org.objectweb.fractal.api.control.LifeCycleController#STOPPED}
   *           state.
   */
  void removeThreads(int i) throws IllegalLifeCycleException;

}