/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.control.activity.task.thread;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.objectweb.dream.control.activity.task.Task;
import org.objectweb.dream.control.activity.task.TaskLifeCycleController;
import org.objectweb.dream.control.activity.task.TaskStoppedListener;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.julia.control.lifecycle.ChainedIllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Basic implementation of a Dream thread.
 * 
 * @see org.objectweb.dream.control.activity.task.thread.AbstractThreadTask
 */
public class ThreadTask extends AbstractThreadTask
    implements
      Runnable,
      TaskLifeCycleController
{
  protected Object  threadTaskLock       = new Object();
  protected Thread  thread               = null;
  protected List    taskStoppedListeners = new ArrayList();

  protected boolean executing;

  // ---------------------------------------------------------------------------
  // Implementation of the AbstractThreadTask abstract methods
  // ---------------------------------------------------------------------------

  /**
   * @see AbstractThreadTask#isExecuting()
   */
  protected boolean isExecuting()
  {
    return executing;
  }

  /**
   * @see AbstractThreadTask#setExecuting(boolean)
   */
  protected void setExecuting(boolean b)
  {
    executing = b;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Runnable interface
  // ---------------------------------------------------------------------------

  /**
   * @see java.lang.Runnable#run()
   */
  public void run()
  {
    logger.log(BasicLevel.DEBUG, "Begin of the run method");
    try
    {
      execute(null);
    }
    catch (InterruptedException e)
    {
      logger.log(BasicLevel.DEBUG, "Interrupted");
    }
    finally
    {
      synchronized (threadTaskLock)
      {
        try
        {
          super.stopFc();
        }
        catch (IllegalLifeCycleException ignored)
        {
          // throws nothing
        }
        Task taskItf = null;
        try
        {
          taskItf = (Task) weaveableC.getFcInterface("task");
        }
        catch (NoSuchInterfaceException ignored)
        {
          // can't happend
        }
        Iterator iter = taskStoppedListeners.iterator();
        while (iter.hasNext())
        {
          ((TaskStoppedListener) iter.next()).taskStopped(taskItf);
          iter.remove();
        }
        thread = null;
        try
        {
          // set component in stopped state.
          super.stopFc();
        }
        catch (IllegalLifeCycleException e1)
        {
          // can't happend.
        }
        logger.log(BasicLevel.DEBUG, "End of the run method");
      }
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the LifeCycleController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
   */
  public void startFc() throws IllegalLifeCycleException
  {
    synchronized (threadTaskLock)
    {
      if (getFcState() == STARTED)
      {
        return;
      }

      // sanity check.
      if (thread != null)
      {
        Error.bug(logger, new IllegalStateException(
            "Thread component is not in start state, but has a thread"));
      }

      super.startFc();
      executing = true;
      thread = new Thread(this);
      logger.log(BasicLevel.DEBUG, "Starting the thread task");
      thread.start();
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
   */
  public void stopFc() throws IllegalLifeCycleException
  {
    Thread t;
    synchronized (threadTaskLock)
    {
      if (getFcState() == STOPPED)
      {
        return;
      }

      // sanity check.
      if (thread == null)
      {
        Error.bug(logger, new IllegalStateException(
            "Thread component is in start state, but has no thread"));
      }

      executing = false;
      logger.log(BasicLevel.DEBUG, "Interrupting the thread task");
      thread.interrupt();
      t = thread;
    }
    try
    {
      logger.log(BasicLevel.DEBUG, "Join thread task");
      t.join();
    }
    catch (InterruptedException e)
    {
      throw new ChainedIllegalLifeCycleException(e, weaveableC,
          "Interrupted while waiting for the end of the thread.");
    }
    logger.log(BasicLevel.DEBUG, "Thread task stop");
  }

  /**
   * @see TaskLifeCycleController#asyncStop(TaskStoppedListener)
   */
  public void asyncStop(TaskStoppedListener listener)
  {
    synchronized (threadTaskLock)
    {
      if (getFcState() == STOPPED)
      {
        //    thread is already stopped
        Task taskItf = null;
        try
        {
          taskItf = (Task) weaveableC.getFcInterface("task");
        }
        catch (NoSuchInterfaceException ignored)
        {
          // can't happend
        }
        listener.taskStopped(taskItf);
        return;
      }

      // sanity check.
      if (thread == null)
      {
        Error.bug(logger, new IllegalStateException(
            "Thread component is in start state, but has no thread"));
      }

      executing = false;
      logger.log(BasicLevel.DEBUG, "Interrupting the thread task");
      taskStoppedListeners.add(listener);
      thread.interrupt();
    }
  }
}