/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message.codec;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.Iterator;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.message.ExtensibleMessage;
import org.objectweb.dream.message.ExtensibleMessageNC;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageAlreadyExistException;
import org.objectweb.dream.message.MessageNC;
import org.objectweb.dream.message.MessageTypeNC;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Abstract implementation of a message codec using data input/output to
 * decode/encode messages. This class implements
 */
public abstract class AbstractMessageCodecDataStream extends AbstractComponent
    implements
      MessageCodec
{

  protected MessageManager messageManagerItf;

  // ---------------------------------------------------------------------------
  // Encoding methods
  // ---------------------------------------------------------------------------

  /**
   * Decode recursively a message and its sub messages (if any). This method use
   * the {@link #doDecodeSingleMessage(DataInput) }method to decode a single
   * message.
   */
  protected Message doDecode(DataInput input) throws IOException
  {
    Message message = doDecodeSingleMessage(input);
    if (message instanceof MessageNC)
    {
      boolean hasSubMessage = input.readBoolean();
      if (hasSubMessage)
      {
        if (!(message instanceof ExtensibleMessage))
        {
          throw new IOException(
              "Invalid decoded message, must implement ExtensibleMessage.");
        }
        // read unnamed sub message.
        ExtensibleMessage extensibleMessage = (ExtensibleMessage) message;
        while (hasSubMessage)
        {
          Message subMessage = doDecode(input);
          extensibleMessage.addSubMessage(subMessage);
          hasSubMessage = input.readBoolean();
        }
      }

      hasSubMessage = input.readBoolean();
      if (hasSubMessage)
      {
        if (!(message instanceof ExtensibleMessageNC))
        {
          throw new IOException(
              "Invalid decoded message, must implements ExtensibleMessageNC.");
        }
        // read named sub message.
        ExtensibleMessageNC extensibleMessageNC = (ExtensibleMessageNC) message;
        while (hasSubMessage)
        {
          String subMessageName = input.readUTF();
          Message subMessage = doDecode(input);
          try
          {
            extensibleMessageNC.addSubMessage(subMessageName, subMessage);
          }
          catch (MessageAlreadyExistException e)
          {
            Error.bug(logger, e);
          }
          hasSubMessage = input.readBoolean();
        }
      }
    }
    else
    {
      boolean hasSubMessage = input.readBoolean();
      if (hasSubMessage)
      {
        if (!(message instanceof ExtensibleMessage))
        {
          throw new IOException(
              "Invalid decoded message, must implement ExtensibleMessage.");
        }
        ExtensibleMessage extensibleMessage = (ExtensibleMessage) message;
        while (hasSubMessage)
        {
          Message subMessage = doDecode(input);
          extensibleMessage.addSubMessage(subMessage);
          hasSubMessage = input.readBoolean();
        }
      }
    }
    return message;
  }

  /**
   * Abstract method used by {@link #doDecode(DataInput) }to decode a single
   * message. This method decode only chunks of message.
   * 
   * @param input the input from which message is decoded.
   * @return a message.
   * @throws IOException if an error occurs.
   */
  protected abstract Message doDecodeSingleMessage(DataInput input)
      throws IOException;

  /**
   * Encode recursively a message and its sub messages (if any). This method use
   * the {@link #doEncodeSingleMessage(Message, DataOutput) }method to encode a
   * single message.
   */
  protected void doEncode(Message message, DataOutput output)
      throws IOException
  {
    doEncodeSingleMessage(message, output);
    if (message instanceof MessageNC)
    {
      MessageNC messageNC = (MessageNC) message;
      Iterator iterator = messageNC.getUnnamedSubMessageIterator();
      while (iterator.hasNext())
      {
        output.writeBoolean(true);
        Message subMessage = (Message) iterator.next();
        doEncode(subMessage, output);
      }
      output.writeBoolean(false);
      MessageTypeNC messageTypeNC = (MessageTypeNC) messageNC.getMessageType();
      iterator = messageTypeNC.getSubMessageNamesIterator();
      while (iterator.hasNext())
      {
        output.writeBoolean(true);
        String subMessageName = (String) iterator.next();
        output.writeUTF(subMessageName);
        doEncode(messageNC.getSubMessage(subMessageName), output);
      }
      output.writeBoolean(false);
    }
    else
    {
      Iterator iterator = message.getSubMessageIterator();
      while (iterator.hasNext())
      {
        output.writeBoolean(true);
        Message subMessage = (Message) iterator.next();
        doEncode(subMessage, output);
      }
      output.writeBoolean(false);
    }
  }

  /**
   * Abstract method used by {@link #doEncode(Message, DataOutput) }to encode a
   * single message. This method encode only chunks of message.
   * 
   * @param message a message to encode.
   * @param output the output to use to encode the message.
   * @throws IOException if an error occurs.
   */
  protected abstract void doEncodeSingleMessage(Message message,
      DataOutput output) throws IOException;

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{MessageManager.ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
    super.bindFc(clientItfName, serverItf);
  }

}