/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.message.manager;

import org.objectweb.dream.message.ChunkType;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageType;

/**
 * A message manager is responsible for the life cycle of messages and chunks.
 * This Interface is a component interface. Each Dream component has a
 * <code>MessageManager</code> client interface (optionally bound).
 */
public interface MessageManager
{

  /** The commonly used name of refer to this interface. */
  String ITF_NAME = "message-manager";

  /**
   * Allocates and returns a message of the specified type. Depending on the
   * implementation, the returned message is extensible (see
   * {@link org.objectweb.dream.message.ExtensibleMessage}) or not.
   * 
   * @param type the type of the message to be created.
   * @return a message.
   * @throws UnknownChunkTypeError if one of the chunk type in the message type
   *           is unknown by the manager (see {@link #createChunk(ChunkType)}).
   */
  Message createMessage(MessageType type) throws UnknownChunkTypeError;

  /**
   * Deletes a message. If the message has been duplicated (by reference), this
   * operation releases the given reference. The message is effectively deleted
   * only if all its references have been released.
   * 
   * @param message the message to be deleted.
   */
  void deleteMessage(Message message);

  /**
   * Duplicates a message. There are two kinds of duplication:
   * <ul>
   * <li>by <b>reference </b>. The returned Message is <i>the same object </i>
   * than the given one. The manager keeps track of the number of message
   * references, so that the {@link #deleteMessage(Message)}operation only
   * deletes the message when all the duplicated references have been released
   * </li>
   * <li>by <b>value </b> (cloning). The returned message is a new message
   * which contains a copy of each chunk and each enclosed message of the
   * original message</li>
   * </ul>
   * 
   * @param message the message to be duplicated.
   * @param clone the duplication mode. <code>false</code> for reference
   *          duplication, <code>true</code> for value duplication.
   * @return the duplicated message.
   */
  Message duplicateMessage(Message message, boolean clone);

  /**
   * Allocates and returns a chunk of the specified type. A given manager
   * implementation knows only a restricted set of {@link ChunkType}, so if the
   * requesed chunk type is unknown, a {@link UnknownChunkTypeError}is thrown
   * 
   * @param type the type of the chunk to be created.
   * @return a chunk.
   * @throws UnknownChunkTypeError if the chunk type can not be resolved by the
   *           manager (cannot find implementation).
   */
  Object createChunk(ChunkType type) throws UnknownChunkTypeError;

  /**
   * Deletes a chunk. If the chunk has been duplicated (by reference), this
   * operation releases the given reference. The chunk is effectively deleted
   * only if all its references have been released.
   * 
   * @param chunk the chunk to delete.
   */
  void deleteChunk(Object chunk);

  /**
   * Duplicates a chunk. As for messages, there are two kinds of duplications
   * (see {@link #duplicateMessage(Message, boolean)}).
   * 
   * @param chunk the chunk to be duplicated.
   * @param clone the duplication mode.
   * @return the duplicated chunk.
   */
  Object duplicateChunk(Object chunk, boolean clone);

  /**
   * Returns the message manager identifier.
   * 
   * @return the message manager identifier.
   */
  short getMessageManagerId();
}